define(["require", "exports", "./transpileHelpers"], function (require, exports, transpileHelpers_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports._getImportIdentifiers = exports._getPackageName = exports._getImports = exports._tryParseExample = exports.tryParseExample = exports.isExampleValid = exports.IMPORT_REGEX = void 0;
    // Don't reference anything importing Monaco in this file to avoid pulling Monaco into the
    // main bundle or breaking tests!
    /**
     * Match an import from a TS file.
     * - Group 1: imported names or \* (empty if side effect-only import)
     * - Group 2: import path without quotes
     * @internal
     */
    exports.IMPORT_REGEX = /^import\s+(?:([^;'"]+?)\s+from\s+)?['"]([^'"]+?)['"];/;
    /** Find the component name to render from a TS file (group 1) */
    var COMPONENT_NAME_REGEX = /^export (?:class|const) (\w+)/m;
    /** Given an import path, find the package name @internal */
    var PACKAGE_NAME_REGEX = /^(@[\w-]+\/[\w-]+|[\w-]+)/;
    /**
     * Match an individual imported item.
     * - Group 1: original item name or *
     * - Group 2: rename, if any
     */
    var IMPORT_ITEM_REGEX = /([\w*]+)(?:\s+as\s+(\w+))?/g;
    /** Matches the React import */
    var REACT_REGEX = /^import \* as React from ['"]react['"];/m;
    /**
     * Determines whether an example is "valid" for purposes of the transform code: it conforms to the
     * expected structure and only contains imports from supported packages.
     *
     * NOTE: You should confirm that the code is syntactically valid before calling this function.
     * If the code is not syntactically valid, this function's behavior is undefined.
     *
     * @param example - Syntactically valid TS code for an example
     * @param supportedPackages - Supported packages for imports (React is implicitly supported)
     * @returns Whether the example is valid
     */
    function isExampleValid(example, supportedPackages) {
        return typeof tryParseExample(example, supportedPackages) !== 'string';
    }
    exports.isExampleValid = isExampleValid;
    /**
     * Determines whether an example is editable and if so, returns the code and the component to render.
     * If it's not editable, returns an error message.
     *
     * NOTE: You should confirm that the code is syntactically valid before calling this function.
     * If the code is not syntactically valid, this function's behavior is undefined (it will likely
     * return incorrect/illogical output).
     *
     * @param example - Syntactically valid TS code for an example
     * @param supportedPackages - Supported packages for imports (React is implicitly supported)
     * @returns Example info if the example is valid, or an error message if not
     */
    function tryParseExample(example, supportedPackages) {
        try {
            return _tryParseExample(example, Object.keys((0, transpileHelpers_1._supportedPackageToGlobalMap)(supportedPackages)));
        }
        catch (ex) {
            return 'Caught error while processing example: ' + ex;
        }
    }
    exports.tryParseExample = tryParseExample;
    /** @internal */
    function _tryParseExample(example, supportedPackages) {
        // Use .source because IE 11 doesn't support creating a regex from a regex
        var possibleComponents = example.match(new RegExp(COMPONENT_NAME_REGEX.source, 'gm'));
        var imports = _getImports(example);
        if (!REACT_REGEX.test(example)) {
            return "The example must include \"import * as React from 'react';\"";
        }
        else if (/^export default/m.test(example)) {
            return '"export default" is not supported by the editor.';
        }
        else if (!possibleComponents || possibleComponents.length > 1) {
            return ('The example must export a single class or const for the component to render ' +
                "(found: ".concat(possibleComponents ? possibleComponents.length : 'none', ")."));
        }
        else if (/require(<.+?>)?\(/m.test(example)) {
            return '"require(...)" is not supported by the editor.';
        }
        else {
            for (var _i = 0, imports_1 = imports; _i < imports_1.length; _i++) {
                var importInfo = imports_1[_i];
                var path = importInfo.path, packageName = importInfo.packageName, items = importInfo.identifiers, text = importInfo.text;
                if (path === 'react') {
                    if (!REACT_REGEX.test(text)) {
                        return "Invalid React import format for the editor. Please only use \"import * as React from 'react'\".";
                    }
                }
                else if (path.indexOf('.scss') !== -1) {
                    return 'Importing scss is not supported by the editor.';
                }
                else if (path[0] === '.') {
                    return 'Relative imports are not supported by the editor.';
                }
                else if (!items.length) {
                    return 'Importing a file for its side effects ("import \'path\'") is not supported by the editor.';
                }
                else if (items[0].name === '*') {
                    return '"import *" is not supported by the editor (except from react).';
                }
                else if (!packageName) {
                    return "Unrecognized import path: \"".concat(path, "\"");
                }
                else if (supportedPackages.indexOf(packageName) === -1) {
                    return "Importing from package \"".concat(packageName, "\" is not supported by the editor.");
                }
                else if (items[0].name === 'default') {
                    return 'Default imports are not supported by the editor.';
                }
                else if (path !== packageName) {
                    var packageRelativePath = path.split(packageName + '/', 2)[1];
                    if (packageRelativePath.split('/').length > 2) {
                        return 'Importing from more than two levels below the package root is not supported by the editor.';
                    }
                }
            }
        }
        return {
            tsCode: example,
            component: example.match(COMPONENT_NAME_REGEX)[1],
            imports: imports,
        };
    }
    exports._tryParseExample = _tryParseExample;
    /** @internal */
    function _getImports(example) {
        // Use .source because IE 11 doesn't support creating a regex from a regex
        var imports = example.match(new RegExp(exports.IMPORT_REGEX.source, 'gm'));
        if (!imports) {
            return [];
        }
        return imports.map(function (importStatement) {
            var importMatch = importStatement.match(exports.IMPORT_REGEX);
            var importPath = importMatch[2];
            return {
                text: importMatch[0],
                packageName: _getPackageName(importPath),
                path: importPath,
                identifiers: _getImportIdentifiers(importMatch[1]),
            };
        });
    }
    exports._getImports = _getImports;
    /** @internal */
    function _getPackageName(path) {
        var packageNameMatch = path.match(PACKAGE_NAME_REGEX) || undefined;
        return (packageNameMatch && packageNameMatch[0]) || '';
    }
    exports._getPackageName = _getPackageName;
    /** @internal */
    function _getImportIdentifiers(contents) {
        // This will be:
        // empty        for `import 'foo'`
        // * as Foo     for `import * as Foo from 'foo'`
        // Foo          for `import Foo from 'foo'`
        // { Foo, Bar } for `import { Foo, Bar } from 'foo'`
        var hadBracket = (contents || '').indexOf('{') !== -1;
        contents = (contents || '').replace('{', '').replace('}', '').trim();
        var items = [];
        IMPORT_ITEM_REGEX.lastIndex = 0;
        var itemMatch = IMPORT_ITEM_REGEX.exec(contents);
        if (itemMatch) {
            if (hadBracket) {
                // named imports
                do {
                    items.push({ name: itemMatch[1], as: itemMatch[2] });
                } while ((itemMatch = IMPORT_ITEM_REGEX.exec(contents)));
            }
            else if (itemMatch[1] === '*') {
                // import *
                items.push({ name: '*', as: itemMatch[2] });
            }
            else {
                // default import
                items.push({ name: 'default', as: itemMatch[1] });
            }
        }
        return items;
    }
    exports._getImportIdentifiers = _getImportIdentifiers;
});
//# sourceMappingURL=exampleParser.js.map