"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transpileAndEval = exports.transpile = void 0;
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var monaco = tslib_1.__importStar(require("@fluentui/monaco-editor"));
var Utilities_1 = require("@fluentui/react/lib/Utilities");
var exampleTransform_1 = require("./exampleTransform");
var transpileHelpers_1 = require("./transpileHelpers");
var win = (0, Utilities_1.getWindow)();
/**
 * Transpile the model's current code from TS to JS.
 */
// This is intentionally not an async function, because debugging within transpiled async functions
// is next to impossible.
function transpile(model) {
    var transpiledOutput = { error: undefined, output: undefined };
    var filename = model.uri.toString();
    return monaco.languages.typescript
        .getTypeScriptWorker()
        .then(function (getWorker) { return getWorker(model.uri); })
        .then(function (worker) {
        return worker.getEmitOutput(filename).then(function (output) {
            // Get diagnostics to find out if there were any syntax errors (there's also getSemanticDiagnostics
            // for type errors etc, but it may be better to allow the user to just find and fix those
            // via intellisense rather than blocking compilation, since they may be non-fatal)
            return worker.getSyntacticDiagnostics(filename).then(function (syntacticDiagnostics) {
                syntacticDiagnostics = syntacticDiagnostics.filter(function (d) { return d.category === 1; } /*error*/);
                if (syntacticDiagnostics.length) {
                    // Don't try to run the example if there's a syntax error
                    transpiledOutput.error = (0, transpileHelpers_1._getErrorMessages)(syntacticDiagnostics, model.getValue());
                }
                else {
                    transpiledOutput.output = output.outputFiles[0].text;
                    if (win && win.transpileLogging) {
                        /* eslint-disable no-console */
                        console.log('TRANSPILED:');
                        console.log(transpiledOutput.output);
                        /* eslint-enable no-console */
                    }
                }
                return transpiledOutput;
            });
        });
    })
        .catch(function (ex) {
        // Log the error to the console so people can see the full stack/etc if they want
        // eslint-disable-next-line no-console
        console.error(ex);
        transpiledOutput.error = ex.message;
        return transpiledOutput;
    });
}
exports.transpile = transpile;
/**
 * Transpiles the code, does an additional transform to prepare for rendering, and evals the code.
 *
 * @param model - Current editor text model
 * @param supportedPackages - Supported packages for imports, grouped by global name
 * (React is implicitly supported)
 * @returns Returns an object with the output string and component to render if successful,
 * or error(s) if unsuccessful.
 */
// This is intentionally not an async function, because debugging within transpiled async functions
// is next to impossible.
function transpileAndEval(model, supportedPackages) {
    var exampleTs = model.getValue();
    return transpile(model)
        .then(function (transpileOutput) {
        if (transpileOutput.error) {
            return transpileOutput;
        }
        /* eslint-disable no-eval */
        var transformedExample = (0, exampleTransform_1.transformExample)({
            tsCode: exampleTs,
            jsCode: transpileOutput.output,
            returnFunction: true,
            supportedPackages: supportedPackages,
        });
        if (transformedExample.output) {
            return tslib_1.__assign(tslib_1.__assign({}, transformedExample), { 
                // Pass in the right React in case there's a different global one on the page...
                component: eval(transformedExample.output)(React) });
        }
        else {
            return { error: transformedExample.error || 'Unknown error transforming example' };
        }
    })
        .catch(function (err) {
        // Log the error to the console so people can see the full stack/etc if they want
        // eslint-disable-next-line no-console
        console.error(err);
        return { error: typeof err === 'string' ? err : err.message };
    });
}
exports.transpileAndEval = transpileAndEval;
//# sourceMappingURL=transpile.js.map