import { __extends } from "tslib";
import * as React from 'react';
import { EditorError } from './EditorError';
/**
 * Error boundary to catch and display any errors thrown while rendering the example component.
 * It also handles displaying errors from transforming the code.
 *
 * The example preview should be rendered as a child of this component. (This component can't just
 * render the preview directly because error boundaries only catch errors in passed-in children.)
 */
var EditorErrorBoundary = /** @class */ (function (_super) {
    __extends(EditorErrorBoundary, _super);
    function EditorErrorBoundary() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        return _this;
    }
    EditorErrorBoundary.getDerivedStateFromError = function (error) {
        return { caughtError: "Error while rendering component: ".concat(error.message || error) };
    };
    EditorErrorBoundary.prototype.componentDidUpdate = function (prevProps, prevState) {
        var _a = this, props = _a.props, state = _a.state;
        // remove the caught error state if:
        // - the error state is not new (present in both curr/prev state)
        // - we have an updated result to render
        if (state.caughtError &&
            prevState.caughtError &&
            props.transformResult.output !== prevProps.transformResult.output) {
            this.setState({ caughtError: undefined });
        }
        if (!(state.caughtError || props.transformResult.error)) {
            // keep track of the last known-good children, to reuse if there's an error
            this._lastGoodChildren = props.children;
        }
    };
    EditorErrorBoundary.prototype.componentDidCatch = function (error, errorInfo) {
        // Log the error to the console so people can see the full stack/etc if they want
        // (only in production, because React logs these errors itself in dev mode)
        if (process.env.NODE_ENV === 'production') {
            /* eslint-disable no-console */
            console.error(error.stack || error);
            console.error('In component: ' + errorInfo.componentStack);
            /* eslint-enable no-console */
        }
        // If there was another error within the past second, get rid of the "last good children"
        // because re-rendering them may cause an infinite loop (has sometimes happened with list
        // examples in the past)
        var errorTime = Date.now();
        if (this._lastErrorTime && errorTime - this._lastErrorTime < 1000) {
            this._lastGoodChildren = null;
        }
        this._lastErrorTime = errorTime;
    };
    EditorErrorBoundary.prototype.render = function () {
        var caughtError = this.state.caughtError;
        var error = caughtError || this.props.transformResult.error;
        return (React.createElement(React.Fragment, null,
            React.createElement(EditorError, { error: error }),
            // If there was an error either transforming or rendering, reuse the most recent
            // successfully-rendered children (especially important if the error is from rendering,
            // because attempting to re-render bad children causes the page to crash)
            error ? this._lastGoodChildren : this.props.children));
    };
    return EditorErrorBoundary;
}(React.Component));
export { EditorErrorBoundary };
//# sourceMappingURL=EditorErrorHandler.js.map